/// Provides mappings from character (string) to hexadecimal value (integer).
/// Hex value represents enabled and disabled segments for given character.
///
/// Example for 7-segment display:
/// `0` DEC = `0x7E` HEX = `1111110` BIN
/// which means that every segment except the middle one is enabled
class CharacterSegmentMap {
  /// Special (dividers) characters.
  static const dividerCharacters = <String, String>{
    'decimalPoint': '.',
    'colon': ':',
  };

  /// Characters that can be displayed on 7-segment display.
  static const seven = <String, int>{
    ' ': 0x00,
    '0': 0x7E,
    '1': 0x30,
    '2': 0x6D,
    '3': 0x79,
    '4': 0x33,
    '5': 0x5B,
    '6': 0x5F,
    '7': 0x70,
    '8': 0x7F,
    '9': 0x7B,
    'A': 0x77,
    'b': 0x1F,
    'C': 0x4E,
    'c': 0x0D,
    'd': 0x3D,
    'E': 0x4F,
    'F': 0x47,
    'G': 0x5E,
    'H': 0x37,
    'I': 0x06,
    'J': 0x3C,
    'L': 0x0E,
    'n': 0x15,
    'O': 0x7E,
    'o': 0x1D,
    'P': 0x67,
    'q': 0x73,
    'r': 0x05,
    'S': 0x5B,
    't': 0x0F,
    'U': 0x3E,
    'u': 0x1C,
    'y': 0x3B,
  };

  /// Characters that can be displayed on 14-segment display.
  static const fourteen = <String, int>{
    ' ': 0x0000,
    '0': 0x3F0C,
    '1': 0x1808,
    '2': 0x36C0,
    '3': 0x3C40,
    '4': 0x19C0,
    '5': 0x2581,
    '6': 0x2FC0,
    '7': 0x3800,
    '8': 0x3FC0,
    '9': 0x3DC0,
    'A': 0x3BC0,
    'B': 0x3C52,
    'C': 0x2700,
    'D': 0x3C12,
    'E': 0x2780,
    'F': 0x2380,
    'G': 0x2F40,
    'H': 0x1BC0,
    'I': 0x2412,
    'J': 0x1E00,
    'K': 0x0389,
    'L': 0x0700,
    'M': 0x1B28,
    'N': 0x1B21,
    'O': 0x3F00,
    'P': 0x33C0,
    'Q': 0x3F01,
    'R': 0x33C1,
    'S': 0x2DC0,
    'T': 0x2012,
    'U': 0x1F00,
    'V': 0x030C,
    'W': 0x1B05,
    'X': 0x002D,
    'Y': 0x1DC0,
    'Z': 0x240C,
    'a': 0x0682,
    'b': 0x0781,
    'c': 0x06C0,
    'd': 0x1C44,
    'e': 0x0684,
    'f': 0x00CA,
    'g': 0x1C48,
    'h': 0x0382,
    'i': 0x0002,
    'j': 0x0214,
    'k': 0x001B,
    'l': 0x0300,
    'm': 0x0AC2,
    'n': 0x0282,
    'o': 0x0EC0,
    'p': 0x03A0,
    'q': 0x1848,
    'r': 0x0280,
    's': 0x0441,
    't': 0x0780,
    'u': 0x0E00,
    'v': 0x0204,
    'w': 0x0A05,
    'x': 0x002D,
    'y': 0x1C50,
    'z': 0x0484,
    '+': 0x00D2,
    '-': 0x00C0,
    '*': 0x00FF,
    '<': 0x0089,
    '>': 0x0064,
    '=': 0x04C0,
    '~': 0x00CC,
    //'!': 0x3001,
    '@': 0x3750,
    '#': 0x1CD2,
    //'?': 0x6085,
    '&': 0x26B1,
    '^': 0x0005,
    '%': 0x09FF,
    '\$': 0x2DD2,
    '(': 0x0009,
    ')': 0x0024,
    '[': 0x2700,
    ']': 0x3C00,
    '{': 0x24A4,
    '}': 0x2449,
    '\\': 0x0021,
    '|': 0x0012,
    '/': 0x000C,
    '_': 0x0400,
    '`': 0x0020,
    '"': 0x1010,
    "'": 0x0010,
    ',': 0x0004,
    //'.': 0x0001,
    //':': 0x0012,
    ';': 0x0014,
  };

  /// Characters that can be displayed on 16-segment display.
  static const sixteen = <String, int>{
    ' ': 0x0000,
    '0': 0xFF0C,
    '1': 0x3008,
    '2': 0xEEC0,
    '3': 0xFC40,
    '4': 0x31C0,
    '5': 0xCD81,
    '6': 0xDFC0,
    '7': 0xF000,
    '8': 0xFFC0,
    '9': 0xFDC0,
    'A': 0xF3C0,
    'B': 0xFC52,
    'C': 0xCF00,
    'D': 0xFC12,
    'E': 0xCF80,
    'F': 0xC380,
    'G': 0xDF40,
    'H': 0x33C0,
    'I': 0xCC12,
    'J': 0x3E00,
    'K': 0x0389,
    'L': 0x0F00,
    'M': 0x3328,
    'N': 0x3321,
    'O': 0xFF00,
    'P': 0xE3C0,
    'Q': 0xFF01,
    'R': 0xE3C1,
    'S': 0xDDC0,
    'T': 0xC012,
    'U': 0x3F00,
    'V': 0x030C,
    'W': 0x3305,
    'X': 0x002D,
    'Y': 0x3DC0,
    'Z': 0xCC0C,
    'a': 0x0E82,
    'b': 0x0782,
    'c': 0x0680,
    'd': 0x3842,
    'e': 0x0684,
    'f': 0x40D2,
    'g': 0x8592,
    'h': 0x0382,
    'i': 0x0002,
    'j': 0x0612,
    'k': 0x001B,
    'l': 0x0300,
    'm': 0x12C2,
    'n': 0x0282,
    'o': 0x0682,
    'p': 0x8390,
    'q': 0x8192,
    'r': 0x0280,
    's': 0x8582,
    't': 0x0780,
    'u': 0x0602,
    'v': 0x0204,
    'w': 0x1205,
    'x': 0x002D,
    'y': 0x3850,
    'z': 0x0484,
    '+': 0x00D2,
    '-': 0x00C0,
    '*': 0x00FF,
    '<': 0x0089,
    '>': 0x0064,
    '=': 0x0CC0,
    '~': 0x00CC,
    //'!': 0x3000,
    '@': 0xEF50,
    '#': 0x3CD2,
    //'?': 0xE042,
    '&': 0x8EB1,
    '^': 0x0005,
    '%': 0x99DE,
    '\$': 0xDDD2,
    '(': 0x0009,
    ')': 0x0024,
    '[': 0x4812,
    ']': 0x8412,
    '{': 0x4892,
    '}': 0x8452,
    '\\': 0x0021,
    '|': 0x0012,
    '/': 0x000C,
    '_': 0x0C00,
    '`': 0x0020,
    '"': 0x2010,
    "'": 0x0010,
    ',': 0x0004,
    //'.': 0x0001,
    //':': 0x0012,
    ';': 0x0014,
  };
}
